@@ -1,3 +1,5 @@
+use crate::optim::OptimizerSession;
+
 use super::lir::{LirScan, LirScanOpParams};
 use tract_data::internal::*;
 
@@ -90,12 +92,15 @@ impl Scan {
 
     fn declutter_body(
         &self,
+        session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
         if !self.decluttered {
             let mut new = self.clone();
-            new.body = self.body.clone().into_decluttered()?;
+            let mut body = self.body.clone();
+            session.optimize(&mut body)?;
+            new.body = body;
             new.decluttered = true;
             Ok(Some(TypedModelPatch::replace_single_op(model, node, &node.inputs, new)?))
         } else {
@@ -105,6 +110,7 @@ impl Scan {
 
     fn declutter_body_axes(
         &self,
+        _session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
@@ -178,6 +184,7 @@ impl Scan {
 
     fn declutter_const_initializer(
         &self,
+        _session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
@@ -210,12 +217,13 @@ impl Scan {
 
     fn declutter_discard_unused_input_mapping(
         &self,
+        _session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
         for (inner_input_id, input) in self.body.input_outlets()?.iter().enumerate() {
             let source_node = self.body.node(input.node);
-            if source_node.outputs[0].successors.len() == 0 {
+            if source_node.outputs[0].successors.len() == 0 && !self.body.output_outlets()?.contains(input) {
                 let mut new_inputs = node.inputs.clone();
                 let slot = match &self.input_mapping[inner_input_id] {
                     InputMapping::Full { slot } => Some(slot),
@@ -257,6 +265,7 @@ impl Scan {
 
     fn declutter_discard_useless_outer_output(
         &self,
+        _session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
@@ -300,6 +309,7 @@ impl Scan {
 
     fn declutter_discard_empty_output_mapping_with_body_output(
         &self,
+        _session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
@@ -322,6 +332,7 @@ impl Scan {
 
     fn declutter_pull_batcheable_input(
         &self,
+        _session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
@@ -403,6 +414,7 @@ impl Scan {
 
     fn declutter_pull_batcheable_output(
         &self,
+        _session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
@@ -742,16 +754,18 @@ impl TypedOp for Scan {
     fn change_axes(
         &self,
         _model: &TypedModel,
-        _node: &TypedNode,
+        node: &TypedNode,
         io: InOut,
         change: &AxisOp,
     ) -> TractResult<Option<AxisChangeConsequence>> {
+        trace!("Entering node {}", node);
         let body_leading_outlet = match io {
             InOut::In(ix) => {
                 if let Some(input) = self.input_mapping.iter().position(|im| im.slot() == Some(ix))
                 {
                     self.body.input_outlets()?[input]
                 } else {
+                    trace!("Node {} blocking (input mapping) ", node);
                     return Ok(None);
                 }
             }
@@ -765,18 +779,26 @@ impl TypedOp for Scan {
             }
         };
         let axis_change = AxisChange { outlet: body_leading_outlet, op: change.clone() };
-        self.try_body_axes_change(axis_change, false)
+        let result = self.try_body_axes_change(axis_change, false)?;
+        if result.is_some() {
+            trace!("{} accepted axis change", node);
+        } else {
+            trace!("{} rejected axis change", node);
+        }
+        Ok(result)
     }
 
-    fn declutter(
+    fn declutter_with_session(
         &self,
+        session: &mut OptimizerSession,
         model: &TypedModel,
         node: &TypedNode,
     ) -> TractResult<Option<TypedModelPatch>> {
         macro_rules! pass {
             ($func:ident) => {
-                if let Some(mut r) =
-                    self.$func(model, node).with_context(|| format!("{}", stringify!($func)))?
+                if let Some(mut r) = self
+                    .$func(session, model, node)
+                    .with_context(|| format!("{}", stringify!($func)))?
                 {
                     trace!(stringify!($func));
                     r.push_context(stringify!($func));
